% =========================================================================
% file Jacobi.m
%
% Scope: This function computes the sequence of polynomials p_k(w), k=0,...,m,
%        that are orthonormal with respect to the Jacobi weight 
%        w(x)=(1-x)^al(1+x)^be using the three-term recurrence formula,
%        and evaluate them on a vector of points.
%        
% Call: J = Jacobi(m,al,be,x)
%
% Input: m - maximum degree of the polynomial sequence
%        al,be - exponents of the Jacobi weight 
%        x - column vector of points at which to evaluate the sequence
%
% Output: J - matrix of size n x (m+1) constructed row by row:
%             the i-th row contains the sequence of polynomials 
%             p_k(w) k=0,...,m evaluated at the point x(i)
%             
% Notes: Requires the functions class1.m and scelta_ic.m to work.
%
% Last Update: 16/01/2025
% =========================================================================

function J = Jacobi(m,al,be,x)

    ic = scelta_ic(al,be);
    [a,b,muzero]=class1(ic, m+1, al, be);
    jac0=(muzero)^(-0.5);
    if m==0
        J=ones(length(x),1)*jac0;
    elseif m==1
        J=zeros(length(x),2);
        J(:,1)=ones(length(x),1)*jac0;
        J(:,2)=((x-a(1))*jac0)/b(1);
    else
        J=zeros(length(x),m+1);
        jac(:,1)=((x-a(1))*jac0)/b(1);
        jac(:,2)=((x-a(2)).*jac(:,1)-b(1)*jac0)/b(2);
        for j=3:m
            jac(:,j)=((x-a(j)).*jac(:,j-1)-b(j-1)*jac(:,j-2))./b(j);
        end 
        J(:,1)=ones(length(x),1)*jac0;
        J(:,2:end)=jac;
    end
    
end