% =========================================================================
% file coefficienti.m
%
% Scope: This function computes the coefficients of the product rule
%        to approximate the Volterra operator V
%
% Call: C = coefficienti(m,x,lam,al,be,rho,sig,y)
%
% Input: m = number of points on which the rule is constructed
%        x = row vector of nodes of the rule 
%        lam = row vector of Christoffel numbers  
%        al,be = esponenti del peso di Jacobi w(x)=(1-x)^al(1+x)^be
%        rho,sig = esponenti del peso di Jacobi w(x)=(1-x)^al(1+x)^be
%        y = column vector of points
%
% Output: C = matrix of size ly x m containing the coefficients of rule.
%             The matrix is built by rows:
%             the i-th row contains the m coefficients evaluated at y(i)
%             i.e. c_1(y(i)),c_2(y(i)),...,c_m(y(i))
%
% Notes: Requires the functions gam.m, gaussq_new.m and Jacobi.m to work
%
% Last Update: 16/01/2025
% =========================================================================

function C = coefficienti(m,x,lam,al,be,rho,sig,y)

    jac=Jacobi(m-1,al,be,x'); 
    ic = scelta_ic(rho,sig);
    [zk,lk]=gaussq_new(ic,floor(m/2)+1,rho,sig,0,0);
    ly = length(y);
    t=gam(y,zk)'; 
    s=zeros(ly,m); 
    for i=1:ly
        s(i,:)=lk*Jacobi(m-1,al,be,t(:,i));
    end 
    s1 = s*jac';
    ky = ((1+y)/2).^(rho+sig+1);
    C = ky.*(s1.*lam); 

end