% =========================================================================
% file main_nystrom.m
%
% Scope: This function computes the approximate solution of second kind 
%        Volterra-Fredholm equation (VFIE) of the form 
%        (I + mu1 V + mu2 K)f(y) = g(y)
%        
% Call: [f,c] = main_nystrom(m,al,be,rho,sig,ga,de,mu1,mu2,y,example)
%
% Input: m - number of points
%        al,be - exponents of the Jacobi weight w(x)=(1-x)^al(1+x)^be (al,be>-1)
%        rho,sig - exponents of the weight (y-x)^rho(1+x)^sig (rho,sig>-1)
%        ga,de - exponents of the Jacobi weight u(x)=(1-x)^ga(1+x)^de (ga,de>=0)
%        mu1 - scalar multiplying the Volterra operator
%        mu2 - scalar multiplying the Fredholm operator
%        y - column vector of points 
%        example - parameter determining the chosen example
%
% Output: f - approximated solution of the equation in the input point y
%         c - condition number in infinity norm of the system
%
% Last Update: 16/01/2025
% =========================================================================

function [f,c] = main_nystrom(m,al,be,rho,sig,ga,de,mu1,mu2,y,example)

    %--------------------Computation of the nodes-----------------------
    ic = scelta_ic(al,be);
    [x,lam]=gaussq_new(ic,m,al,be,0,0);
    
    %-------------Diagonal matrix of the space weight u-----------------
    U=diag(weight(ga,de,x));
    
    %-----------------------Right-hand side-----------------------------
    b=U*righthand(example,x');
    
    %---------Matrix of coefficients for Fredholm operator--------------
    K = kernelk(example,x,x'); 

    %-----------Diagonal matrix of the Christoffel numbers--------------
    L = diag(lam); 
    
    %---------Matrix of coefficient for Volterra operator---------------
    H = kernelh(example,x,x'); 
    Cx = coefficienti(m,x,lam,al,be,rho,sig,x');
    
    %---------------Coefficients matrix of the system-------------------
    A=mu1*Cx.*H+mu2*K*L;
    B=eye(m)+U*A/U;
    
    %----------Condition number and solution of the system-------------
    c=cond(B,inf);
    sol=B\b;
    
    %--------------------Nystrom interpolant---------------------------
    uu = weight(ga,de,y);
    Ky = kernelk(example,x,y)/U;
    Hy = kernelh(example,x,y)/U;
    Cy = coefficienti(m,x,lam,al,be,rho,sig,y);
    by = righthand(example,y);
    f = (by-(mu1*Cy.*Hy+mu2*Ky*L)*sol).*uu;

end
