% =========================================================================
% file class1.m
%
% Scope: This subroutine supplies the coefficients a_j, b_j of the recurrence relation
%
%        b_j p_j (x) = (x - a_j) p_{j-1}(x) - b_{j-1} p_{j-2}(x)
%
%        for the various classical (normalized) orthogonal polynomials,
%        and the zero-th moment
%
%        muzero = integral w(x) dx
%
%        of the given polynomial's weight function w(x).
%        Since the polynomials are orthonormalized, the tridiagonal matrix is
%        guaranteed to be symmetric.
%
% Call: [a,b,muzero] = class1(kind,n,al,be)
%
% Input: kind - an integer between 1 and 6 giving the type of rule
%             kind = 1 -> Legendre weight w(x)=1 (al=be=0)
%             kind = 2 -> 1st kind Chebyshev weight w(x)=1/sqrt(1-x^2) (al=be=-1/2)
%             kind = 3 -> 2nd kind Chebyshev weight w(x)=sqrt(1-x^2) (al=be=1/2)
%             kind = 4 -> Hermite weight w(x)=e^(-x^2)
%             kind = 5 -> Jacobi weight w(x)=(1-x)^alpha(1+x)^beta (al,be>-1)
%             kind = 6 -> Laguerre weight w(x)=x^alpha e^(-x) (al>-1)      
%        n - degree of the polynomial
%        al - real parameter used only for Jacobi and Laguerre polynomials
%        be - real parameter used only for Jacobi polynomials
%
% Output: a - row vector containing the coefficients a_j
%         b - row vector containing the coefficients b_j
%         muzero - scalar containing the zero-th moment
% 
% Notes: The computation of the Laguerre and Jacobi polynomials
%        requires the Matlab built-in function gamma.
%
% Last Update: 16/01/2025
% =========================================================================

function [a,b,muzero] = class1(kind,n,al,be)
    
    a = zeros(1,n);
    b = zeros(1,n-1);
    nm1 = n - 1 ;
    if kind ==1 % Legendre polynomials on (-1,1)
    muzero = 2.0;
    for   i = 1: nm1
        a(i) = 0.0;
        abi = i;
        b(i) = abi/sqrt(4*abi*abi - 1.0);
    end 
    a(n) = 0.0;
    elseif kind==2 % 1st kind Chebyshev polynomials on (-1,1)
      muzero = pi;
      for   i = 1: nm1
         a(i) = 0.0;
         b(i) = 0.5;
      end 
      b(1) = sqrt(0.5);
      a(n) = 0.0;
    elseif kind==3 % 2nd kind Chebyshev polynomials on (-1,1)
      muzero = pi/2.0;
      for  i = 1 : nm1
         a(i) = 0.0 ;
         b(i) = 0.5;
      end 
      a(n) = 0.0;
    elseif kind==4 % Hermite polynomials on (-inf,+inf)
      muzero = sqrt(pi);
      for i = 1 : nm1
         a(i) = 0.0;
         b(i) = sqrt(i/2);
      end 
      a(n) = 0.0;
    elseif kind==5 % Jacobi polynomials on (-1,1)
      ab = al + be;
      abi = 2.0 + ab;
      muzero = 2.0^(ab+1.)*gamma(al+1)*gamma(be+1) / gamma(abi);
      a(1) = (be - al)/abi;
      b(1) = sqrt(4.0*(1.0 + al)*(1.0 + be)/((abi + 1.0)*abi*abi));
      a2b2 = be*be - al*al;
      for  i = 2: nm1
         abi = 2.0*i + ab;
         a(i) = a2b2/((abi - 2.0)*abi);
         b(i) = sqrt(4*i*(i+al)*(i+be)*(i+ab)/((abi*abi-1)*abi*abi));
      end 
      abi = 2.0*n + ab;
      a(n) = a2b2/((abi - 2.0)*abi);
    else % kind=6 Laguerre polynomials on (0, +inf)
      muzero = gamma(al + 1.0);
      for   i = 1 :nm1
         a(i) = 2.0*i - 1.0 + al;
         b(i) =  sqrt(i*(i + al));
      end 
      a(n) = 2.0*n - 1 + al;
    end 

end
