% =========================================================================
% file gaussq_new.m
%
% Scope: This function computes the nodes and weights
%        for Gaussian quadrature rules.
%
% Call: [d,z] = gaussq_new(ic,n,al,be,kpts,endpts)
%
% Input: ic - an integer between 1 and 6 giving the type of rule
%             ic = 1 -> Legendre weight w(x)=1 (al=be=0)
%             ic = 2 -> 1st kind Chebyshev weight w(x)=1/sqrt(1-x^2) (al=be=-1/2)
%             ic = 3 -> 2nd kind Chebyshev weight w(x)=sqrt(1-x^2) (al=be=1/2)
%             ic = 4 -> Hermite weight w(x)=e^(-x^2)
%             ic = 5 -> Jacobi weight w(x)=(1-x)^alpha(1+x)^beta (al,be>-1)
%             ic = 6 -> Laguerre weight w(x)=x^alpha e^(-x) (al>-1)      
%        n - number of points used for the quadrature rule
%        al - real parameter used only for Gauss-Jacobi and Gauss-Laguerre rules
%        be - real parameter used only for Gauss-Jacobi rules
%        kpts - integer normally equal to 0, unless the left or right endpoint
%               (or both) of the interval is required to be a node (this is called 
%               Gauss-Radau or Gauss-Lobatto quadrature).
%               In this case kpts is the number of fixed endpoints (1 or 2).
%        endpts - real array of length 2 that contains the values of
%                 any fixed endpoints (only if kpts is equal to 1 or 2).
%
% Output: d - row vector of length n containing the quadrature nodes
%         z - row vector of length n containing the quadrature weights
%             
% Notes: This function is a translation of the original Fortran gaussq function
%
% References: - G.H. Golub, J.H. Welsch. "Calculation of Gaussian quadrature rules".
%               Mathematics of Computation 23 (1969), pp. 221-230.
%             - G.H. Golub. "Some modified matrix eigenvalue problems".
%               SIAM Review 15 (1973), pp. 318-334.
%             - A.H. Stroud, D. Secrest. "Gaussian quadrature formulas". 
%               Prentice-Hall, Englewood Cliffs, N.J. (1966)
%
% Last Update: 16/01/2025
% =========================================================================

function [d,z] = gaussq_new(ic,n,al,be,kpts,endpts)

    if any(ic==(1:6))
        [a1,b1,mu1]=class1(ic,n,al,be);
        if kpts == 1
           [gam]=solve(endpts(1),n,a1,b1);
           a1(n)=gam*b1(n-1)^2+endpts(1);
        elseif kpts==2
           [gam]=solve(endpts(1),n,a1,b1);
           [gam1]=solve(endpts(2),n,a1,b1);
           t1=((endpts(1)-endpts(2))/(gam1-gam));
           b1(n-1)=sqrt(t1);
           a1(n)=gam*t1+endpts(1);
        end
        [d,z,~]=gausq2(n,a1,b1);
        z=z.^2*mu1;
    end
    
    if ic==7
        n1=fix(n/2);
        if rem(n,2)==0
           alpha1=(al-1)*0.5 ;
        else
           alpha1=(al+1)*0.5; 
        end
       [a1,b1,mu1]=class1(6,n1,alpha1,0);
       [dp,zp,~]=gausq2(n1,a1,b1);
       dp1=sqrt(dp);
       zp=zp.^2*mu1;
       if rem(n,2)==0
           d=sort(cat(2,dp1,-dp1));
           z=[fliplr(0.5*zp),0.5*zp];
       else
           d=sort(cat(2,dp1,0,-dp1));
           z1= 0.5*zp./(dp);
           lambda0=gamma(alpha1)-2*sum(z1);
           z=[fliplr(z1),lambda0,z1];
       end
    end

end


function[d,z,ierr]= gausq2(n,d,e)

% -------------------------------------------------------------------------
% This subroutine is a translation of an Algol procedure,
% Num. Math. 12, pp. 377-383 (1968) by Martin and Wilkinson,
% as modified in Num. Math. 15, 450 (1970) by Dubrulle.
% This subroutine finds the eigenvalues and first components of the
% eigenvectors of a symmetric tridiagonal matrix by the implicit ql method.
%
% Input: n - order of the matrix;
%        d - contains the diagonal elements of the input matrix;
%        e - contains the subdiagonal elements of the input matrix
%            in its first n-1 positions.  e(n) is arbitrary;
%        z - contains the first row of the identity matrix.
%
% Output: d - contains the eigenvalues in ascending order. If an
%             error exit is made, the eigenvalues are correct but
%             unordered for indices 1, 2, ..., ierr-1;
%         z - contains the first components of the orthonormal eigenvectors
%             of the symmetric tridiagonal matrix. If an error exit is
%             made, z contains the eigenvectors associated with the stored
%             eigenvalues;
%         ierr - is set to zero for normal return, j if the j-th eigenvalue
%                has not been determined after 30 iterations.
% -------------------------------------------------------------------------

      z=zeros(1,n); 
      z(1)=1;
      ierr = 0;
      if  n~=1
          e(n) = 0;
          for l = 1:n
            j = 0;
            % look for small sub-diagonal element
            ivar =1;
            while ivar ==1
                for  m = l : n
                    if m == n
                       break
                    end
                    if abs(e(m)) <= eps * (abs(d(m)) + abs(d(m+1)))
                       break
                    end
                end 
                p = d(l);
                if  m ~= l
                    if j == 30
                        ierr=l;
                        return,
                    end
                    j = j + 1;
                    % form shift 
                    g = (d(l+1) - p) / (2 * e(l));
                    r = sqrt(g*g+1);
                    if g  < 0
                        ggg=-abs(r);
                    else
                        ggg=abs(r);
                    end
                    g = d(m) - p + e(l) / (g + ggg);
                    s = 1;
                    c = 1;
                    p = 0;
                    mml = m - l;
                    % for i=m-1 step -1 until l do...
                    for ii = 1: mml 
                        i = m - ii;
                        f = s * e(i);
                        b = c * e(i);
                        if abs(f) >= abs(g)
                            c = g / f;
                            r = sqrt(c*c+1.0);
                            e(i+1) = f * r;
                            s = 1.0 / r;
                            c = c * s;
                        else
                            s = f / g;
                            r = sqrt(s*s+1.0);
                            e(i+1) = g * r;
                            c = 1.0 / r;
                            s = s * c;
                        end
                        g = d(i+1) - p;
                        r = (d(i) - g) * s + 2.0 * c * b;
                        p = s * r;
                        d(i+1) = g + p;
                        g = c * r - b;
                        % form first component of vector
                        f = z(i+1);
                        z(i+1) = s * z(i) + c * f;
                        z(i) = c * z(i) - s * f;
                    end 
                    d(l) = d(l) - p;
                    e(l) = g;
                    e(m) = 0.0;
                else
                    ivar =2;
                end 
            end
          end
          % order eigenvalues and eigenvectors
          for ii = 2 : n
              i = ii - 1;
              k = i;
              p = d(i);
              for  j = ii : n
                  if d(j) <= p
                      k = j;
                      p = d(j);
                  end                
              end 
              if k ~= i
                   d(k) = d(i);
                   d(i) = p;
                   p = z(i);
                   z(i) = z(k);
                   z(k) = p;
              end 
          end
          % set error - no convergence to an eigenvalue after 30 iterations
      end 

end


function[a,b,muzero]=class1(kind,n,al,be)

% -------------------------------------------------------------------------
% This subroutine supplies the coefficients a_j, b_j of the recurrence relation
%
% b_j p_j (x) = (x - a_j) p_{j-1}(x) - b_{j-1} p_{j-2}(x)
%
% for the various classical (normalized) orthogonal polynomials,
% and the zero-th moment
%
% muzero = integral w(x) dx
%
% of the given polynomial's weight function w(x).
% Since the polynomials are orthonormalized, the tridiagonal matrix is
% guaranteed to be symmetric.
%
% Input: kind - an integer between 1 and 6 giving the type of rule
%             kind = 1 -> Legendre weight w(x)=1 (al=be=0)
%             kind = 2 -> 1st kind Chebyshev weight w(x)=1/sqrt(1-x^2) (al=be=-1/2)
%             kind = 3 -> 2nd kind Chebyshev weight w(x)=sqrt(1-x^2) (al=be=1/2)
%             kind = 4 -> Hermite weight w(x)=e^(-x^2)
%             kind = 5 -> Jacobi weight w(x)=(1-x)^alpha(1+x)^beta (al,be>-1)
%             kind = 6 -> Laguerre weight w(x)=x^alpha e^(-x) (al>-1)      
%        n - degree of the polynomial
%        al - real parameter used only for Jacobi and Laguerre polynomials
%        be - real parameter used only for Jacobi polynomials
%
% Output: a - row vector containing the coefficients a_j
%         b - row vector containing the coefficients b_j
%         muzero - scalar containing the zero-th moment
% 
% Notes: The computation of the Laguerre and Jacobi polynomials
%        requires the Matlab built-in function gamma.
% -------------------------------------------------------------------------
    
    a = zeros(1,n);
    b = zeros(1,n-1);
    nm1 = n - 1 ;
    if kind ==1 % Legendre polynomials on (-1,1)
    muzero = 2.0;
    for   i = 1: nm1
        a(i) = 0.0;
        abi = i;
        b(i) = abi/sqrt(4*abi*abi - 1.0);
    end 
    a(n) = 0.0;
    elseif kind==2 % 1st kind Chebyshev polynomials on (-1,1)
      muzero = pi;
      for   i = 1: nm1
         a(i) = 0.0;
         b(i) = 0.5;
      end 
      b(1) = sqrt(0.5);
      a(n) = 0.0;
    elseif kind==3 % 2nd kind Chebyshev polynomials on (-1,1)
      muzero = pi/2.0;
      for  i = 1 : nm1
         a(i) = 0.0 ;
         b(i) = 0.5;
      end 
      a(n) = 0.0;
    elseif kind==4 % Hermite polynomials on (-inf,+inf)
      muzero = sqrt(pi);
      for i = 1 : nm1
         a(i) = 0.0;
         b(i) = sqrt(i/2);
      end 
      a(n) = 0.0;
    elseif kind==5 % Jacobi polynomials on (-1,1)
      ab = al + be;
      abi = 2.0 + ab;
      muzero = 2.0^(ab+1.)*gamma(al+1)*gamma(be+1) / gamma(abi);
      a(1) = (be - al)/abi;
      b(1) = sqrt(4.0*(1.0 + al)*(1.0 + be)/((abi + 1.0)*abi*abi));
      a2b2 = be*be - al*al;
      for  i = 2: nm1
         abi = 2.0*i + ab;
         a(i) = a2b2/((abi - 2.0)*abi);
         b(i) = sqrt(4*i*(i+al)*(i+be)*(i+ab)/((abi*abi-1)*abi*abi));
      end 
      abi = 2.0*n + ab;
      a(n) = a2b2/((abi - 2.0)*abi);
    else % kind=6 Laguerre polynomials on (0, +inf)
      muzero = gamma(al + 1.0);
      for   i = 1 :nm1
         a(i) = 2.0*i - 1.0 + al;
         b(i) =  sqrt(i*(i + al));
      end 
      a(n) = 2.0*n - 1 + al;
    end 

end


